@@ -4,8 +4,8 @@
 
 ;; Author: Md Arif Shaikh <arifshaikh.astro@gmail.com>
 ;; Homepage: https://github.com/md-arif-shaikh/tzc
-;; Package-Version: 0.3.0
-;; Package-Revision: 8f425cd6f020
+;; Package-Version: 20260205.416
+;; Package-Revision: 99d0420aeed0
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: convenience
 
@@ -31,12 +31,14 @@
 ;; time-zone to a list of favourite time-zones.
 ;;
 ;; A list of favourite time zones could be set using like following
-;; (setq tzc-favourite-time-zones-alist '(("Asia/Kolkata" "Kolkata") ("America/New_York" "New York") ("Europe/Berlin" "Berlin")))
+;; (setq tzc-favourite-time-zones-alist '(("Asia/Kolkata" "Kolkata")
+;; ("America/New_York" "New York") ("Europe/Berlin" "Berlin")))
 
 ;;; Code:
 (require 'timezone)
 (require 'subr-x)
 (require 'org)
+(require 'org-element)
 
 (defvar tzc-color--time-zone-label "#98C379"
   "Color to indicate a time zone label.")
@@ -109,13 +111,14 @@
   "Get the list of favourite time zones."
   (mapcar #'car tzc-favourite-time-zones-alist))
 
-(defun tzc--get-time-zone-label (time-zone)
-  "Get the label for the TIME-ZONE."
-  (cond ((member time-zone (tzc--favourite-time-zones))
-	 (nth 1 (assoc time-zone tzc-favourite-time-zones-alist)))
-	((string-match-p "/" time-zone)
-	 (string-replace "_" " " (nth 1 (split-string time-zone "/"))))
-	(t time-zone)))
+(defun tzc--closest-string (target candidates)
+  "Return closest string in CANDIDATES to TARGET."
+  (car
+   (sort candidates
+         (lambda (a b)
+           (< (string-distance target a)
+              (string-distance target b))))))
+
 
 (defcustom tzc-main-dir (cond ((string-equal system-type "darwin") "/usr/share/zoneinfo.default/")
 			      ((string-equal system-type "gnu/linux") "/usr/share/zoneinfo/"))
@@ -125,7 +128,7 @@
 
 (defcustom tzc-areas '("Africa" "America" "Antarctica" "Arctic" "Asia" "Atlantic" "Australia" "Brazil" "Canada" "Chile" "Europe" "Indian" "Mexico" "Pacific" "US")
   "Areas to look for the timezone info."
-  :type 'list
+  :type '(repeat string)
   :group 'tzc)
 
 (defun tzc--get-time-zones ()
@@ -139,9 +142,24 @@
 
 (defcustom tzc-time-zones (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones)))
   "List of time zones."
-  :type 'list
+  :type '(repeat string)
   :group 'tzc)
 
+(defun tzc--get-time-zone-label (time-zone)
+  "Get the label for the TIME-ZONE."
+  (cond
+   ((null time-zone) "Local Time")
+   ((member time-zone (tzc--favourite-time-zones))
+    (nth 1 (assoc time-zone tzc-favourite-time-zones-alist)))
+   ((string-match-p "\\`[A-Za-z]+\\'" time-zone)
+    (user-error "%s is not a valid timezone.  Should be in the format Area/City!" time-zone))
+   ((string-match-p "/" time-zone)
+    (if (member time-zone tzc-time-zones)
+	(string-replace "_" " " (nth 1 (split-string time-zone "/")))
+      (user-error "%s is not a recognized timezone.  Perhaps looking for %s!" time-zone
+		  (tzc--closest-string time-zone tzc-time-zones))))
+   (t time-zone)))
+
 (defcustom tzc-world-clock-buffer-name "*tzc-wclock*"
   "Name of the `tzc-world-clock' buffer."
   :type 'string
@@ -161,7 +179,7 @@
 (defun tzc--+-p (timeshift)
   "Check if the TIMESHIFT in contain +- string."
   (when (stringp timeshift)
-    (or (string-match-p "+\d{4}" timeshift) (string-match-p "-\d{4}" timeshift))))
+    (string-match-p "\\`\\(?:[A-Z]+\\)?[-+][0-9]\\{2,4\\}\\'" timeshift)))
 
 (defun tzc--get-offset (time-zone &optional date)
   "Get the time offset for TIME-ZONE on a given DATE."
@@ -253,7 +271,7 @@ erroneous calculation.  Please use correct format for time!"))
     (append (cons time-now time-list-after) time-list-before)))
 
 ;;;###autoload
-(defun tzc-convert-time (time-string from-zone to-zone from-date)
+(defun tzc-convert-time (time-string from-zone to-zone &optional from-date)
   "Convert a given time as given in TIME-STRING from FROM-ZONE to TO-ZONE.
 Optionally on a given FROM-DATE."
   (interactive
@@ -302,17 +320,63 @@ The conversion is computed for the given FROM-DATE."
     (align-regexp (point-min) (point-max) "\\(\\s-*\\)=")
     (switch-to-buffer-other-window tzc-world-clock-buffer-name)))
 
+(defun tzc--time-zone-format-error ()
+"Error message to display for invalid timezone format."
+(user-error
+ "Invalid timezone format.
+Use Area/City (e.g. Europe/London) or an offset such as UTC+0530 or GMT-0400!"))
+
+(defun tzc--extract-timezone (timestamp)
+  "Return timezone token from Org TIMESTAMP, or nil."
+  (when (string-match "\\s-+\\([^ >]+\\)>\\'" timestamp)
+    (match-string 1 timestamp)))
+
 (defun tzc--get-zoneinfo-from-time-stamp (timestamp)
-  "Get the zoneinfo Area/City from TIMESTAMP."
-  (when (string-match "[a-z]+[/][a-z]+" timestamp)
-    (match-string 0 timestamp)))
+  "Get the zoneinfo from TIMESTAMP."
+  (let ((tz (tzc--extract-timezone timestamp)))
+    (cond
+     ((null tz) nil)
+     ((string-match-p "\\`[A-Za-z]+/[A-Za-z_]+\\'" tz)
+      (if (member tz tzc-time-zones)
+          tz
+        (user-error "%s is not a valid timezone.  Perhaps looking for %s?"
+                    tz (tzc--closest-string tz tzc-time-zones))))
+     ((string-match-p "\\`\\(?:[A-Z]+\\)?[-+][0-9]\\{2,4\\}\\'" tz)
+      tz)
+     ((string-match-p "\\`[A-Z]\\{2,5\\}\\'" tz)
+      (tzc--time-zone-format-error))
+     (t (tzc--time-zone-format-error)))))
+
+(defun tzc--get-timestamp-at-point ()
+  "Return Org timestamp at point as (STRING BEGIN END)."
+  (let* ((pos (point))
+         (ctx (org-element-context))
+         ts)
+    ;; Case 1: real timestamp element
+    (setq ts (org-element-lineage ctx '(timestamp) t))
+
+    ;; Case 2: planning timestamps — choose by point location
+    (when (and (null ts)
+               (eq (org-element-type ctx) 'planning))
+      (dolist (prop '(:scheduled :deadline :closed))
+        (let ((p (org-element-property prop ctx)))
+          (when (and p
+                     (<= (org-element-property :begin p) pos)
+                     (>= (org-element-property :end p) pos))
+            (setq ts p)))))
+    (when ts
+      (list
+       (org-element-property :raw-value ts)
+       (org-element-property :begin ts)
+       (org-element-property :end ts)))))
 
 ;;;###autoload
 (defun tzc-convert-time-at-mark (to-zone)
-  "Convert time at the marked region to TO-ZONE."
+  "Convert time at point to TO-ZONE."
   (interactive
    (list (completing-read "Enter To Zone:  " (tzc--get-time-zones))))
-  (let* ((timestamp (buffer-substring-no-properties (mark) (point)))
+  (let* ((timestamp (or (nth 0 (tzc--get-timestamp-at-point))
+                        (error "No timestamp found at point!")))
 	 (parsed-list (parse-time-string timestamp))
 	 (from-zone)
 	 (hour)
@@ -337,13 +401,21 @@ erroneous calculation.  Please use correct format for time!")
     (tzc-convert-time (format "%02d:%02d" hour minute) from-zone to-zone (format "%04d-%02d-%02d" year month day))))
 
 (defun tzc-convert-and-replace-time-at-mark (to-zone)
-  "Convert time at the marked region to TO-ZONE."
+  "Convert time at point to TO-ZONE and replace it."
   (interactive
    (list (completing-read "Enter To Zone:  " (tzc--get-time-zones))))
-  (let* ((converted-time-strings (split-string (tzc-convert-time-at-mark to-zone) " = "))
-	 (converted-time (nth 1 converted-time-strings)))
-    (kill-region (mark) (point))
-    (insert converted-time)))
+  (let* ((timestamp-details (tzc--get-timestamp-at-point))
+	 (beg)
+	 (end))
+    (if timestamp-details
+	(setq beg (nth 1 timestamp-details)
+	      end (nth 2 timestamp-details))
+      (user-error "No org timestamp found at point!"))
+    (let* ((converted-time-strings
+	    (split-string (tzc-convert-time-at-mark to-zone) " = "))
+           (converted-time (nth 1 converted-time-strings)))
+      (delete-region beg end)
+      (insert converted-time))))
 
 (define-derived-mode tzc-world-clock-mode special-mode "tzc world clock"
   "Major mode for buffer that displays times in various time zones.
@@ -445,12 +517,13 @@ See `tzc-world-clock'."
 
 ;;;; convert org time-stamp
 ;;;###autoload
-(defun tzc-convert-org-time-stamp-at-mark (to-zone)
-  "Convert `org-time-stamp` at the marked region to TO-ZONE."
+(defun tzc-convert-org-time-stamp (timestamp to-zone)
+  "Convert TIMESTAMP to TO-ZONE."
   (interactive
-   (list (completing-read "Enter To Zone:  " (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones))))))
-  (let* ((timestamp (buffer-substring-no-properties (mark) (point)))
-	 (from-zone-exists-p (tzc--get-zoneinfo-from-time-stamp timestamp))
+   (list
+    (read-string "Enter timestamp to convert: ")
+    (completing-read "Enter To Zone:  " (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones))))))
+  (let* ((from-zone-exists-p (tzc--get-zoneinfo-from-time-stamp timestamp))
 	 (from-zone (if from-zone-exists-p
 			from-zone-exists-p
 		      (completing-read "No Time Zone info found in the time stamp. Enter Time Zone of the current time stamp in Area/City format:  " (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones))))))
@@ -468,7 +541,8 @@ See `tzc-world-clock'."
 	 (shift (cond ((equal day-shift 1) "++1")
 		      ((equal day-shift -1) "--1")
 		      (t "++0")))
-	 (converted-date (org-read-date nil nil shift nil (org-time-string-to-time (format "%02d-%02d-%02d" year month day))))
+	 (converted-date (format-time-string "%F"
+			  (org-read-date nil t shift nil (org-time-string-to-time (format "%04d-%02d-%02d" year month day)))))
 	 (start-bracket (cond ((string-match-p "<" timestamp) "<")
 			      ((string-match-p "\\[" timestamp) "[")
 			      (t "")))
@@ -480,13 +554,31 @@ See `tzc-world-clock'."
 	     (if from-zone-exists-p (concat " " to-zone) "") end-bracket)))
 
 ;;;###autoload
-(defun tzc-convert-and-replace-org-time-stamp-at-mark (to-zone)
-  "Convert `org-time-stamp` at the marked region to TO-ZONE."
+(defun tzc-convert-org-time-stamp-at-mark (to-zone)
+  "Convert `org-time-stamp` at point to TO-ZONE."
   (interactive
    (list (completing-read "Enter To Zone:  " (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones))))))
-  (let* ((converted-time-stamp (tzc-convert-org-time-stamp-at-mark to-zone)))
-    (kill-region (mark) (point))
-    (insert converted-time-stamp)))
+  (let* ((timestamp (or (nth 0 (tzc--get-timestamp-at-point))
+                        (error "No org timestamp found at point!"))))
+    (tzc-convert-org-time-stamp timestamp to-zone)))
+
+;;;###autoload
+(defun tzc-convert-and-replace-org-time-stamp-at-mark (to-zone)
+  "Convert `org-time-stamp` at point to TO-ZONE and replace it."
+  (interactive
+   (list (completing-read "Enter To Zone:  " (delete-dups (append (tzc--favourite-time-zones) (tzc--get-time-zones))))))
+  (let* ((timestamp-details (tzc--get-timestamp-at-point))
+	 (timestamp)
+	 (beg)
+	 (end))
+    (if timestamp-details
+	(setq timestamp (nth 0 timestamp-details)
+	      beg (nth 1 timestamp-details)
+	      end (nth 2 timestamp-details))
+      (user-error "No org timestamp found at point!"))
+    (let* ((converted-time-stamp (tzc-convert-org-time-stamp timestamp to-zone)))
+      (delete-region beg end)
+      (insert converted-time-stamp))))
 
 ;;;###autoload
 (defun tzc-get-time-shift-between-zones (from-zone to-zone from-date)
